@@ -669,4 +669,10 @@ def timedExec(updateFunc: t.Union[t.Generator, t.Callable], interval: int,
       timer.stop()
   timer.timeout.connect(update)
   timer.start(interval)
-  return timer
\ No newline at end of file
+  return timer
+
+def gracefulNext(generator: t.Generator):
+  try:
+    return next(generator)
+  except StopIteration as ex:
+    return ex.value
\ No newline at end of file
