@@ -15,6 +15,8 @@ from functools import wraps
 
 from typing_extensions import Protocol
 
+from utilitys.fns import gracefulNext
+
 __all__ = ['ActionStack', 'Action', 'DeferredActionStackMixin']
 
 # _generatorCallable = Callable[[...], Union[Generator, Any]]
@@ -94,12 +96,6 @@ class Appendable(Protocol):
   def append(self):
     raise NotImplementedError
 
-def gracefulNext(generator: t.Generator):
-  try:
-    return next(generator)
-  except StopIteration as ex:
-    return ex.value
-
 class ActionStack:
   """ The main undo stack.
 
