@@ -447,6 +447,7 @@ class ParamEditor(QtWidgets.QDockWidget):
                    nest=True,
                    parentParam: Parameter=None,
                    returnParam=False,
+                   container: ParamContainer=None,
                    **kwargs):
     """
     Like `registerProp`, but for functions instead along with interactive parameters
@@ -476,7 +477,11 @@ class ParamEditor(QtWidgets.QDockWidget):
       directly to the parent parameter specified by `namePath` + `baseRegisterPath`
     :param returnParam: Whether to return the parent parameter associated with this newly
       registered function
-    :param parentParam: Which parent should hold this registered function. If None, defaults to `self.param`
+    :param parentParam: Which parent should hold this registered function. If None, defaults to `self.param
+    :param container: Container in which to put child parameters from the registered function. This has a similar effect
+      to calling `registerProps` on each of the function arguments with this container. *Note*: while the former will
+      register props according to the `constParam` PrjParam, this will register to the container using the raw parameter
+      name. A KeyError is raised when a new parameter name clashes with something already in the container.
     :param kwargs: All additional kwargs are passed to AtomicProcess when wrapping the function.
     """
     if not isinstance(func, ProcessStage):
@@ -522,6 +527,12 @@ class ParamEditor(QtWidgets.QDockWidget):
       self.sigChangesApplied.connect(runProc)
     self.procToParamsMapping[proc] = parentParam
 
+    if container is not None:
+      for pName in proc.input.hyperParamKeys:
+        if pName in container.keys():
+          raise KeyError(f'{pName} already exists in this container')
+        container[pName] = parentParam.child(pName)
+
     if returnParam:
       return proc, parentParam
     return proc
