@@ -102,8 +102,7 @@ class PrjParam:
     # sufficient to form a proper hash
     return hash(self.name,)
 
-  def toNumeric(self, data: Sequence, offset:bool=None, rescale=False,
-                returnMapping=False):
+  def toNumeric(self, data: Sequence, rescale=False, returnMapping=False):
     """
     Useful for converting string-like or list-like parameters to integer representations.
     If self's `value` is non-numeric data (e.g. strings):
@@ -112,14 +111,7 @@ class PrjParam:
        - If no limits exist, unique values in the input data will be considered the
          limits
     :param data: Array-like data to be turned into numeric values according to the parameter type
-    :param offset: Whether to offset entries (increment all numeric outputs by 1). This
-      is useful when 0 is indicative of e.g. a background label, but also the numeric
-      value corresponding to the first possible set entry. If `offset` is *True*, output
-      values will be incremented by 1 as described. If *False*, they won't. If *None*,
-      offset will be applied depending on the parameter type. If already numeric, no offset
-      will be applied. Otherwise, `offset` will be *True*.
-    :param rescale: If *True*, values will be rescaled to the range 0-1. `offset` is ignored
-      in that case.
+    :param rescale: If *True*, values will be rescaled to the range 0-1
     :param returnMapping: Whether to also return an array of the unique values within `data`.
       If data is already numeric, *None* is returned instead of this array.
    """
@@ -130,8 +122,6 @@ class PrjParam:
         return numericVals, mapping
       return numericVals
     if not np.issubdtype(type(self.value), np.number):
-      if offset is None:
-        offset = True
       # Check for limits that indicate the exhaustive list of possible values.
       # Otherwise, just use the unique values of this set as the limits
       if 'limits' in self.opts:
@@ -151,14 +141,9 @@ class PrjParam:
       # Dummy mapping to allow ops
       class mapping:
         index = np.array([numericVals.max(), numericVals.min()])
-    if offset:
-      numericVals += 1
-      mapping.index += 1
     if rescale:
       min_, max_ = mapping.index.min(), mapping.index.max()
-      if offset:
-        # Make sure to reduce min to account for this
-        min_ -= 1
+      # Make sure to reduce min to account for offset in reverse direction
       numericVals = _rescale(numericVals, min_, max_)
       mapping.index = _rescale(mapping.index, min_, max_)
     if returnMapping:
