@@ -8,6 +8,7 @@ from functools import wraps
 from warnings import warn
 
 import numpy as np
+import pyqtgraph as pg
 from pyqtgraph.Qt import QtCore
 
 from .fns import nameFormatter
@@ -120,8 +121,8 @@ class ProcessStage(ABC):
   disabled = False
   cacheOnDisable = False
   result: ProcessIO = None
-  mainResultKeys: StrList = None
-  mainInputKeys: StrList = None
+  inMap: t.Union[t.List[str], t.Dict[str, str]] = None
+  outMap: t.Union[t.List[str], t.Dict[str, str]] = None
 
   class _DUPLICATE_INFO: pass
   """Identifies information that is the same in two contiguous stages"""
@@ -138,7 +139,7 @@ class ProcessStage(ABC):
     return repr(self)
 
   def __eq__(self, other: NestedProcess):
-    return self.saveState(True, True) == other.saveState(True, True)
+    return pg.eq(self.saveState(True, True), other.saveState(True, True))
 
   def __hash__(self):
     return id(self)
@@ -185,13 +186,17 @@ class ProcessStage(ABC):
       record the `disabled` property if it is *False*. This is an easy way of ignoring default properties. Setting a key
       to *None* will ignore that property entirely.
     """
-    metaProps = ('allowDisable', 'disabled')
+    metaProps = ('allowDisable', 'disabled', 'inMap', 'outMap')
+    class UNSET: pass
     # Convert back if no props were added
     out = {}
     for k in metaProps:
       selfVal = getattr(self, k)
-      cmp = filterOpts.get(k)
-      if cmp is None or np.array_equal(cmp, selfVal):
+      cmp = filterOpts.get(k, UNSET)
+      if (cmp is None
+          or pg.eq(cmp, selfVal)
+        or pg.eq(getattr(type(self), k, UNSET), selfVal)
+      ):
         continue
       out[k] = selfVal
     return out
@@ -217,24 +222,26 @@ class AtomicProcess(ProcessStage):
   be assigned to that result.
   """
 
-  def __init__(self, func: t.Callable, name:str=None, *, needsWrap=False,
-               mainResultKeys: StrList=None, mainInputKeys: StrList=None,
-               docFunc: t.Callable=None,
-               **procIoKwargs):
+  def __init__(self, func: t.Callable, name:str=None, *,
+               inMap=None, outMap=None,
+               wrapWith=None, docFunc: t.Callable=None, **procIoKwargs):
     """
     :param func: Function to wrap
     :param name: Name of this process. If `None`, defaults to the function name with
       camel case or underscores converted to title case.
-    :param needsWrap: For functions not defined by the user, it is often inconvenient if they have
-    to be redefined just to return a FRProcessIO object. If `func` does not return a `FRProcessIO`
-    object, `needsWrap` can be set to `True`. In this case, `func` is assumed to
+    :param wrapWith: For functions not defined by the user, it is often inconvenient if they have
+    to be redefined just to return a ProcessIO object. If `func` does not return a `ProcessIO`
+    object, `wrapWith` can be set to the desired output keys. In this case, `func` is assumed to
     returns either one result or a list of results. It is converted into a function
-    returning a FRProcessIO object instead. Each `mainResultKey` is assigned to each output
-    of the function in order. If only one main result key exists, then the output of the
-    function is assumed to be that key. I.e. in the case where `len(cls.mainResultKeys) == 1`,
+    returning a ProcessIO object instead. Each `wrapWith` key is assigned to each output
+    of the function in order. If only one wrap key exists, then the output of the
+    function is assumed to be that key. I.e. in the case where `len(cls.inMap) == 1`,
     the output is expected to be the direct result, not a sequence of results per key.
-    :param mainResultKeys: Set by parent process as needed
-    :param mainInputKeys: Set by parent process as needed
+    :param inMap: Method of mapping inputs from a previous process into inputs consumable by this one. Dict in the form
+      of {outKey: inExpr}, where inExpr is any valid python expression consisting only of expected inputs from a previous
+      process. If *None*, no mapping is performed.
+    :param outMap: See `inMap`. This is the same concept, but maps as {exposed: resultExpr}, where exposed is what
+      processes after this see, and resultExpr is some expression based just on results from the function eval.
     :param docFunc: Sometimes, `func` is a wrapper around a different function, where this other function is what should
       determine the created user parameters. In these cases (i.e. one function directly calling another), that inner
       function can be provided here for parsing the docstring and parameters. This is only used for the purpose of
@@ -243,10 +250,8 @@ class AtomicProcess(ProcessStage):
     """
     if name is None:
       name = nameFormatter((docFunc or func).__name__)
-    if mainResultKeys is not None:
-      self.mainResultKeys = mainResultKeys
-    if mainInputKeys is not None:
-      self.mainInputKeys = mainInputKeys
+    self.inMap = inMap or self.inMap
+    self.outMap = outMap or self.outMap
 
     self.name = name
     try:
@@ -260,27 +265,25 @@ class AtomicProcess(ProcessStage):
     self.fnDoc = (docFunc or func).__doc__
     """Record function documentation of proxy if needed so wrapper functions properly"""
 
-    if mainInputKeys is not None:
+    if self.inMap is not None:
       keys = set(self.input.keys())
-      missingKeys = set(mainInputKeys) - keys
+      missingKeys = set(self.inMap) - keys
       if missingKeys:
         raise KeyError(f'{name} input signature is missing the following required input keys:\n'
                                 f'{missingKeys}')
 
-    if needsWrap:
-      func = self._wrappedFunc(func, self.mainResultKeys)
+    if wrapWith is not None:
+      func = self._wrappedFunc(func, wrapWith)
     self.func = func
     self.defaultInput = self.input.copy()
 
   @classmethod
-  def _wrappedFunc(cls, func, mainResultKeys: StrList=None):
+  def _wrappedFunc(cls, func, mainResultKeys: StrList):
     """
     Wraps a function returining either a result or list of results, instead making the
     return value an `FRProcessIO` object where each `cls.mainResultkey` corresponds
     to a returned value
     """
-    if mainResultKeys is None:
-      mainResultKeys = cls.mainResultKeys
     if len(mainResultKeys) == 1:
       @wraps(func)
       def newFunc(*args, **kwargs):
@@ -291,6 +294,29 @@ class AtomicProcess(ProcessStage):
         return ProcessIO(**{k: val for k, val in zip(mainResultKeys, func(*args, **kwargs))})
     return newFunc
 
+  def _maybeWrapStageFunc(self, oldresult: t.Any):
+    """
+    It is not possible to determine what value type is returned until a stage is first run. For cases in which this
+    result is *not* a ProcessIO object, but one is required for mapping / pipelining, this function both wraps the
+    old result in a ProcessIO object *and* wraps the atomic function so all future results are also ProcessIO objects.
+    If this process fails, nothing happens to the stage's function
+    """
+    oldFunc = self.func
+    useKeys = list(self.outMap)
+    try:
+      self.func = self._wrappedFunc(oldFunc, useKeys)
+      # Capture current result as a process io
+      def dummyFunc():
+        return oldresult
+      oldresult = self._wrappedFunc(dummyFunc, useKeys)()
+    except Exception as ex:
+      # Many things can go wrong, safely revert
+      warn(f'Tried to convert outputs for process {self.name} into a ProcessIO'
+           f' but this failed with the following error:\n'
+           f'{ex}', UserWarning)
+      self.func = oldFunc
+    return oldresult
+
   @property
   def keysFromPrevIO(self):
     return self.input.keysFromPrevIO
@@ -299,7 +325,10 @@ class AtomicProcess(ProcessStage):
     disable = disable or self.disabled
     self.updateInput(io, **runKwargs)
     if not disable:
-      self.result = self.func(**self.input)
+      out = self.func(**self.input)
+      if self.outMap and not isinstance(out, ProcessIO):
+        out = self._maybeWrapStageFunc(out)
+      self.result = self.computeArgMapping(self.outMap, out)
     elif not self.cacheOnDisable:
       self.result = None
     out = self.result
@@ -307,12 +336,31 @@ class AtomicProcess(ProcessStage):
       out = self.input
     return out
 
+  def computeArgMapping(self, mapping: t.Union[t.Dict[str, str], t.Sequence[str]], toMap: dict, copy=True):
+    """
+    Converts toMap into outputs by rules specified in mapping. e.g. if mapping was {in: out + 5} then `toMap` is searched
+    for "out", the expression is evaluated, and the result is assigend to key "in" in the output.
+    If `mapping` is a sequence instead of dict, those are considered keys in the preferred order. *copy* determines
+    whether the pre-mapped values should all be copied into the result, where keys present in mapping will be overridden
+    """
+    if mapping is None:
+      return toMap
+    out = toMap.copy() if copy else {}
+    if isinstance(mapping, t.Sequence):
+      return ProcessIO(**{kk: toMap[kk] for kk in mapping})
+    # Eval needs dict
+    toMap = dict(toMap)
+    for kk, expr in mapping.items():
+      out[kk] = eval(expr, toMap)
+    return ProcessIO(**out)
+
   def updateInput(self, prevIo: ProcessIO=None, graceful=False, allowExtra=False, **kwargs):
     allowExtra = allowExtra or self.hasKwargs
     if prevIo is None:
       prevIo = ProcessIO()
     useIo = prevIo.copy()
     useIo.update(kwargs)
+    useIo = self.computeArgMapping(self.inMap, useIo)
     selfFmtToUnfmt = {k.lstrip('_'): k for k in self.input}
     requiredKeyFmt = {k for k, v in selfFmtToUnfmt.items() if v in self.input.keysFromPrevIO}
     prevIoKeyToFmt = {k.lstrip('_'): k for k in {**useIo}}
@@ -357,28 +405,19 @@ class AtomicProcess(ProcessStage):
 
 class NestedProcess(ProcessStage):
 
-  def __init__(self, name: str=None, mainInputKeys: StrList=None, mainResultKeys: StrList=None):
+  def __init__(self, name: str=None):
     self.stages: t.List[ProcessStage] = []
     self.name = name
     self.allowDisable = True
     self.result = ProcessIO()
-    if mainInputKeys is not None:
-      self.mainInputKeys = mainInputKeys
-    if mainResultKeys is not None:
-      self.mainResultKeys = mainResultKeys
 
-  def addFunction(self, func: t.Callable, keySpec: t.Union[t.Type[NestedProcess], NestedProcess]=None, **kwargs):
+  def addFunction(self, func: t.Callable, **kwargs):
     """
     Wraps the provided function in an AtomicProcess and adds it to the current process.
     :param func: Forwarded to AtomicProcess
     :param kwargs: Forwarded to AtomicProcess
-    :param keySpec: This argument should have 'mainInputKeys' and 'mainResultKeys' that are used
-      when adding a function to this process. This can be beneficial when an Atomic Process
-      is added with different keys than the current process type
     """
-    if keySpec is None:
-      keySpec = self
-    atomic = AtomicProcess(func, mainResultKeys=keySpec.mainResultKeys, mainInputKeys=keySpec.mainInputKeys, **kwargs)
+    atomic = AtomicProcess(func, **kwargs)
     numSameNames = sum(
         atomic.name == stage.name.split('#')[0] for stage in self)
     if numSameNames > 0:
@@ -437,28 +476,12 @@ class NestedProcess(ProcessStage):
           ex.args = (ex.args,)
         ex.args = (stage,) + ex.args
         raise
-      if not isinstance(newIo, ProcessIO) and isinstance(stage, AtomicProcess):
-        # Try wrapping the inner function for future runs
-        newIo = self._maybeWrapStageFunc(stage, newIo)
       if isinstance(newIo, ProcessIO):
         _activeIo.update(newIo)
 
     self.result = _activeIo
     return self.result
 
-  def _maybeWrapStageFunc(self, stage: AtomicProcess, oldresult: t.Any):
-    oldFunc = stage.func
-    useKeys = stage.mainResultKeys or self.mainResultKeys
-    try:
-      stage.func = stage._wrappedFunc(oldFunc, useKeys)
-      # Capture current result as a process io
-      def dummyFunc():
-        return oldresult
-      oldresult = stage._wrappedFunc(dummyFunc, useKeys)()
-    except Exception:
-      stage.func = oldFunc
-    return oldresult
-
   def saveState(self, includeDefaults=False, includeMeta=False, **metaFilterOpts):
     stageStates = []
     for stage in self:
@@ -479,7 +502,7 @@ class NestedProcess(ProcessStage):
 
   def flatten(self, copy=True, includeDisabled=True):
     if copy:
-      outProc = type(self)(self.name, self.mainInputKeys, self.mainResultKeys)
+      outProc = type(self)(self.name)
     else:
       outProc = self
     outProc.stages = self._getFlatStages(includeDisabled)
@@ -534,11 +557,11 @@ class NestedProcess(ProcessStage):
     for stage in self._nonDisabledStages_flattened():
       res = stage.result
       if not isinstance(res, ProcessIO): continue
-      if any(k not in res for k in self.mainResultKeys):
+      if any(k not in res for k in self.inMap):
         # Missing required keys, not sure how to turn into summary info. Skip
         continue
       if 'summaryInfo' not in res:
-        defaultSummaryInfo = {k: res[k] for k in self.mainResultKeys}
+        defaultSummaryInfo = {k: res[k] for k in self.outMap}
         defaultSummaryInfo.update(name=stage.name)
         res['summaryInfo'] = defaultSummaryInfo
       if res['summaryInfo'] is None:
